@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # The MIT License
 
-# Copyright (c) 2017 - 2018 Tammo Ippen, tammo.ippen@posteo.de
+# Copyright (c) 2017 - 2022 Tammo Ippen, tammo.ippen@posteo.de
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -24,17 +24,22 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 # THE SOFTWARE.
 
 from ._canvas import Canvas
-from ._colors import color
+from ._cmaps import Colormap, ListedColormap
+from ._colors import color, hsl
 from ._figure import Figure
-from ._graphs import hist, histogram, plot, scatter
+from ._graphs import hist, hist_aggregated, histogram, plot, scatter
 
 
 __all__ = [
     'Canvas',
     'color',
+    'Colormap',
     'Figure',
     'hist',
+    'hist_aggregated',
     'histogram',
+    'hsl',
+    'ListedColormap',
     'plot',
     'scatter',
 ]
