@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # The MIT License
 
-# Copyright (c) 2017 - 2018 Tammo Ippen, tammo.ippen@posteo.de
+# Copyright (c) 2017 - 2022 Tammo Ippen, tammo.ippen@posteo.de
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,12 +23,12 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+import colorsys
+import os
 import sys
 
-import six
 
-
-def color(text, fg=None, bg=None, mode='names', no_color=False):
+def color(text, fg=None, bg=None, mode='names', no_color=False, full_reset=True):  # noqa: C901 complex (12)
     """Surround `text` with control characters for coloring
 
     c.f. http://en.wikipedia.org/wiki/ANSI_escape_code
@@ -64,12 +64,20 @@ def color(text, fg=None, bg=None, mode='names', no_color=False):
     If you want to use colorama (https://pypi.python.org/pypi/colorama), you should
     also stick to the ISO 6429 colors.
 
+    The environment variables `NO_COLOR` (https://no-color.org/) and `FORCE_COLOR`
+    (only toggle; see https://nodejs.org/api/tty.html#tty_writestream_getcolordepth_env)
+    have some influence on color output.
+
+    If you do not run in a TTY, e.g. pipe to some other program or redirect output
+    into a file, color codes are stripped as well.
+
     Parameters:
         text: str        Some text to surround.
         fg: multiple     Specify the foreground / text color.
         bg: multiple     Specify the background color.
         color_mode: str  Specify color input mode; 'names' (default), 'byte' or 'rgb'
         no_color: bool   Remove color optionally. default=False
+        full_reset: bool Reset all codes or only color codes. default=True
 
     Returns:
         str: `text` enclosed with corresponding coloring controls
@@ -77,7 +85,19 @@ def color(text, fg=None, bg=None, mode='names', no_color=False):
     if fg is None and bg is None:
         return text
 
-    if not _isatty() or no_color:
+    if no_color or os.environ.get('NO_COLOR'):
+        #  https://no-color.org/
+        return text
+
+    # similar to https://nodejs.org/api/tty.html#tty_writestream_getcolordepth_env
+    # except for only on or of
+    force_color = os.environ.get('FORCE_COLOR')
+    if force_color:
+        force_color = force_color.strip().lower()
+        if force_color in ('0', 'false', 'none'):
+            return text
+
+    if not force_color and not _isatty():
         # only color if tty (not a redirect / pipe)
         return text
 
@@ -87,19 +107,75 @@ def color(text, fg=None, bg=None, mode='names', no_color=False):
     elif mode == 'byte':
         start = _byte(fg, bg)
     elif mode == 'rgb':
-        if isinstance(fg, six.string_types):
+        if isinstance(fg, str):
             fg = _hex2rgb(fg)
-        if isinstance(bg, six.string_types):
+        if isinstance(bg, str):
             bg = _hex2rgb(bg)
 
         start = _rgb(fg, bg)
     else:
         raise ValueError('Invalid mode "{}". Use one of "names", "byte" or "rgb".'.format(mode))
 
-    if start:
-        return start + text + '\x1b[0m'
+    assert start
+    res = start + text
+    if full_reset:
+        return res + '\x1b[0m'
+    else:
+        return res + '\x1b[39;49m'
 
-    # should not be reachable
+
+def hsl(hue, saturation, lightness):
+    """Convert HSL color space into RGB color space.
+
+    In contrast to colorsys.hls_to_rgb, this works directly in
+    360 deg Hue and give RGB values in the range of 0 to 255.
+
+    Parameters:
+        hue: float         Position in the spectrum. 0 to 360.
+        saturation: float  Color saturation. 0 to 1.
+        lightness: float   Color lightness. 0 to 1.
+    """
+    assert 0 <= hue <= 360
+    assert 0 <= saturation <= 1
+    assert 0 <= lightness <= 1
+
+    r, g, b = colorsys.hls_to_rgb(hue / 360.0, lightness, saturation)
+    return int(round(r * 255)), int(round(g * 255)), int(round(b * 255))
+
+
+def rgb2byte(r, g, b):
+    """Convert RGB values into an index for the byte color-mode.
+
+    Parameters:
+        r: int    Red value. Between 0 and 255.
+        g: int    Green value. Between 0 and 255.
+        b: int    Blue value. Between 0 and 255.
+
+    Returns
+        idx: int  Index of approximate color in the byte color-mode.
+    """
+    assert 0 <= r <= 255
+    assert 0 <= g <= 255
+    assert 0 <= b <= 255
+
+    if r == g == b < 244:
+        # gray:
+        gray_idx = _value_to_index(min(238, r), off=8, steps=10)
+        return gray_idx + 232
+
+    # here we also have some gray values ...
+    r_idx = _value_to_index(r)
+    g_idx = _value_to_index(g)
+    b_idx = _value_to_index(b)
+
+    return 16 + 36 * r_idx + 6 * g_idx + b_idx
+
+
+def _value_to_index(v, off=55, steps=40):
+    idx = (v - off) / steps
+    if idx < 0:
+        return 0
+    return int(round(idx))
 
 
 def _isatty():
@@ -137,17 +213,17 @@ def _byte(fg, bg):
 
     fg_ = ''
     if fg is not None:
-        fg_ = '38;5;' + six.text_type(fg)
+        fg_ = '38;5;' + str(fg)
     bg_ = ''
     if bg is not None:
-        bg_ = '48;5;' + six.text_type(bg)
+        bg_ = '48;5;' + str(bg)
 
     return _join_codes(fg_, bg_)
 
 
 def _hex2rgb(h):
     """Transform rgb hex representation into rgb tuple of ints representation"""
-    assert isinstance(h, six.string_types)
+    assert isinstance(h, str)
     if h.lower().startswith('0x'):
         h = h[2:]
     if len(h) == 3:
@@ -170,10 +246,10 @@ def _rgb(fg, bg):
 
     fg_ = ''
     if fg is not None:
-        fg_ = '38;2;' + ';'.join(map(six.text_type, fg))
+        fg_ = '38;2;' + ';'.join(map(str, fg))
     bg_ = ''
     if bg is not None:
-        bg_ = '48;2;' + ';'.join(map(six.text_type, bg))
+        bg_ = '48;2;' + ';'.join(map(str, bg))
 
     return _join_codes(fg_, bg_)
 
@@ -204,6 +280,14 @@ _BACKGROUNDS = {
     'bright_magenta': '105',
     'bright_cyan': '106',
     'bright_white': '107',
+    'bright_black_old': '1;40',
+    'bright_red_old': '1;41',
+    'bright_green_old': '1;42',
+    'bright_yellow_old': '1;43',
+    'bright_blue_old': '1;44',
+    'bright_magenta_old': '1;45',
+    'bright_cyan_old': '1;46',
+    'bright_white_old': '1;47',
 }
 
 _FOREGROUNDS = {
@@ -215,12 +299,20 @@ _FOREGROUNDS = {
     'magenta': '35',
     'cyan': '36',
     'white': '37',
-    'bright_black': '1;30',
-    'bright_red': '1;31',
-    'bright_green': '1;32',
-    'bright_yellow': '1;33',
-    'bright_blue': '1;34',
-    'bright_magenta': '1;35',
-    'bright_cyan': '1;36',
-    'bright_white': '1;37',
+    'bright_black': '90',
+    'bright_red': '91',
+    'bright_green': '92',
+    'bright_yellow': '93',
+    'bright_blue': '94',
+    'bright_magenta': '95',
+    'bright_cyan': '96',
+    'bright_white': '97',
+    'bright_black_old': '1;30',
+    'bright_red_old': '1;31',
+    'bright_green_old': '1;32',
+    'bright_yellow_old': '1;33',
+    'bright_blue_old': '1;34',
+    'bright_magenta_old': '1;35',
+    'bright_cyan_old': '1;36',
+    'bright_white_old': '1;37',
 }
