@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # The MIT License
 
-# Copyright (c) 2017 - 2018 Tammo Ippen, tammo.ippen@posteo.de
+# Copyright (c) 2017 - 2022 Tammo Ippen, tammo.ippen@posteo.de
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,17 +23,15 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-from collections import namedtuple
 from datetime import timedelta
 from itertools import cycle
 import os
 
-from six.moves import zip
-
 from ._canvas import Canvas
-from ._colors import color
+from ._colors import color, rgb2byte
+from ._figure_data import Heat, Histogram, Plot, Span, Text
 from ._input_formatter import InputFormatter
-from ._util import hist, mk_timedelta, timestamp
+from ._util import mk_timedelta, timestamp
 
 # TODO documentation!!!
 # TODO tests
@@ -58,13 +56,13 @@ class Figure(object):
         x_label, y_label: str Define the X / Y axis label.
     """
     _COLOR_SEQ = [
-        {'names': 'white', 'rgb': (255, 255, 255), 'byte': 0X7},
-        {'names': 'red', 'rgb': (255, 0, 0), 'byte': 0x1},
-        {'names': 'green', 'rgb': (0, 255, 0), 'byte': 0x2},
-        {'names': 'yellow', 'rgb': (255, 255, 0), 'byte': 0x3},
-        {'names': 'blue', 'rgb': (0, 0, 255), 'byte': 0x4},
-        {'names': 'magenta', 'rgb': (255, 0, 255), 'byte': 0x5},
-        {'names': 'cyan', 'rgb': (0, 255, 255), 'byte': 0x6},
+        {'names': 'white', 'rgb': (255, 255, 255), 'byte': rgb2byte(255, 255, 255)},
+        {'names': 'red', 'rgb': (255, 0, 0), 'byte': rgb2byte(255, 0, 0)},
+        {'names': 'green', 'rgb': (0, 255, 0), 'byte': rgb2byte(0, 255, 0)},
+        {'names': 'yellow', 'rgb': (255, 255, 0), 'byte': rgb2byte(255, 255, 0)},
+        {'names': 'blue', 'rgb': (0, 0, 255), 'byte': rgb2byte(0, 0, 255)},
+        {'names': 'magenta', 'rgb': (255, 0, 255), 'byte': rgb2byte(255, 0, 255)},
+        {'names': 'cyan', 'rgb': (0, 255, 255), 'byte': rgb2byte(0, 255, 255)},
     ]
 
     def __init__(self):
@@ -75,14 +73,20 @@ class Figure(object):
         self._x_max = None
         self._y_min = None
         self._y_max = None
-        self._color_mode = None
+        self._color_kwargs = {'mode': 'names'}
         self._with_colors = True
         self._origin = True
         self.linesep = os.linesep
         self.background = None
         self.x_label = 'X'
         self.y_label = 'Y'
+        # min, max -> value
+        self.y_ticks_fkt = None
+        self.x_ticks_fkt = None
         self._plots = []
+        self._texts = []
+        self._spans = []
+        self._heats = []
         self._in_fmt = InputFormatter()
 
     @property
@@ -111,9 +115,7 @@ class Figure(object):
 
     @property
     def color_mode(self):
-        if self._color_mode is not None:
-            return self._color_mode
-        return 'names'
+        return self._color_kwargs['mode']
 
     @color_mode.setter
     def color_mode(self, value):
@@ -121,10 +123,21 @@ class Figure(object):
             raise ValueError('Only supports: names, byte, rgb!')
         if self._plots != []:
             raise RuntimeError('Change color mode only, when no plots are prepared.')
-        self._color_mode = value
+        self._color_kwargs['mode'] = value
+
+    @property
+    def color_full_reset(self):
+        return self._color_kwargs.get('full_reset', True)
+
+    @color_full_reset.setter
+    def color_full_reset(self, value):
+        if not isinstance(value, bool):
+            raise ValueError('Only supports bool.')
+        self._color_kwargs['full_reset'] = value
 
     @property
     def with_colors(self):
+        """Whether to plot with or without color."""
         return self._with_colors
 
     @with_colors.setter
@@ -135,6 +148,7 @@ class Figure(object):
 
     @property
     def origin(self):
+        """Show or not show the origin in the plot."""
         return self._origin
 
     @origin.setter
@@ -144,21 +158,48 @@ class Figure(object):
         self._origin = value
 
     def register_label_formatter(self, type_, formatter):
+        """Register a formatter for labels of a certain type.
+
+        See `plotille._input_formatter` for examples.
+
+        Parameters
+        ----------
+        type_
+            A python typte, that can be used for isinstance tests.
+        formatter: (val: type_, chars: int, delta, left: bool = False) -> str
+            Function that formats `val` into a string.
+            chars: int => number of chars you should fill
+            delta      => the difference between the smallest and largest X/Y value
+            left: bool => align left or right.
+        """
         self._in_fmt.register_formatter(type_, formatter)
 
     def register_float_converter(self, type_, converter):
+        """Register a converter from some type_ to float.
+
+        See `plotille._input_formatter` for examples.
+
+        Parameters
+        ----------
+        type_
+            A python type, that can be used for isinstance tests.
+        formatter: (val: type_) -> float
+            Function that formats `val` into a float.
+        """
         self._in_fmt.register_converter(type_, converter)
 
     def x_limits(self):
         return self._limits(self._x_min, self._x_max, False)
 
     def set_x_limits(self, min_=None, max_=None):
+        """Set min and max X values for displaying."""
         self._x_min, self._x_max = self._set_limits(self._x_min, self._x_max, min_, max_)
 
     def y_limits(self):
         return self._limits(self._y_min, self._y_max, True)
 
     def set_y_limits(self, min_=None, max_=None):
+        """Set min and max Y values for displaying."""
         self._y_min, self._y_max = self._set_limits(self._y_min, self._y_max, min_, max_)
 
     def _set_limits(self, init_min, init_max, min_=None, max_=None):
@@ -186,7 +227,7 @@ class Figure(object):
             return low_set, high_set
 
         low, high = None, None
-        for p in self._plots:
+        for p in self._plots + self._texts:
             if is_height:
                 _min, _max = _limit(p.height_vals())
             else:
@@ -207,10 +248,18 @@ class Figure(object):
         else:
             y_delta = delta / self.height
 
-        res = [self._in_fmt.fmt(i * y_delta + ymin, abs(ymax - ymin), chars=10) + ' | '
-               for i in range(self.height)]
+        res = []
+        for i in range(self.height):
+            value = i * y_delta + ymin
+            if self.y_ticks_fkt:
+                value = self.y_ticks_fkt(value, value + y_delta)
+            res += [self._in_fmt.fmt(value, abs(ymax - ymin), chars=10) + ' | ']
+
         # add max separately
-        res += [self._in_fmt.fmt(self.height * y_delta + ymin, abs(ymax - ymin), chars=10) + ' |']
+        value = self.height * y_delta + ymin
+        if self.y_ticks_fkt:
+            value = self.y_ticks_fkt(value, value + y_delta)
+        res += [self._in_fmt.fmt(value, abs(ymax - ymin), chars=10) + ' |']
 
         ylbl = '({})'.format(label)
         ylbl_left = (10 - len(ylbl)) // 2
@@ -230,42 +279,200 @@ class Figure(object):
             starts = ['-' * 11 + '|-', ' ' * 11 + '| ']
         res = []
 
-        res += [starts[0] + '|---------' * (self.width // 10) + '|-> (' + label + ')']
-        res += [starts[1] + ' '.join(self._in_fmt.fmt(i * 10 * x_delta + xmin, delta, left=True, chars=9)
-                                     for i in range(self.width // 10 + 1))]
+        res += [
+            starts[0]
+            + '|---------' * (self.width // 10)
+            + '|'
+            + '-' * (self.width % 10)
+            + '-> (' + label + ')',
+        ]
+        bottom = []
+
+        for i in range(self.width // 10 + 1):
+            value = i * 10 * x_delta + xmin
+            if self.x_ticks_fkt:
+                value = self.x_ticks_fkt(value, value + x_delta)
+            bottom += [self._in_fmt.fmt(value, delta, left=True, chars=9)]
+
+        res += [starts[1] + ' '.join(bottom)]
         return res
 
     def clear(self):
+        """Remove all plots, texts and spans from the figure."""
         self._plots = []
+        self._texts = []
+        self._spans = []
+        self._heats = []
 
-    def plot(self, X, Y, lc=None, interp='linear', label=None):
+    def plot(self, X, Y, lc=None, interp='linear', label=None, marker=None):
+        """Create plot with X , Y values.
+
+        Parameters:
+            X: List[float]     X values.
+            Y: List[float]     Y values. X and Y must have the same number of entries.
+            lc: multiple       The line color.
+            interp: str        The interpolation method. (None or 'linear').
+            label: str         The label for the legend.
+            marker: str        Instead of braille dots set a marker char.
+        """
         if len(X) > 0:
             if lc is None:
                 lc = next(self._color_seq)[self.color_mode]
-            self._plots += [Plot.create(X, Y, lc, interp, label)]
+            self._plots += [Plot(X, Y, lc, interp, label, marker)]
 
-    def scatter(self, X, Y, lc=None, label=None):
+    def scatter(self, X, Y, lc=None, label=None, marker=None):
+        """Create a scatter plot with X , Y values
+
+        Parameters:
+            X: List[float]     X values.
+            Y: List[float]     Y values. X and Y must have the same number of entries.
+            lc: multiple       The line color.
+            label: str         The label for the legend.
+            marker: str        Instead of braille dots set a marker char.
+        """
         if len(X) > 0:
             if lc is None:
                 lc = next(self._color_seq)[self.color_mode]
-            self._plots += [Plot.create(X, Y, lc, None, label)]
+            self._plots += [Plot(X, Y, lc, None, label, marker)]
 
     def histogram(self, X, bins=160, lc=None):
+        """Compute and plot the histogram over X.
+
+        Paramaters:
+            X: List[float]     X values.
+            bins: int          The number of bins to put X entries in (columns).
+            lc: multiple       The line color.
+        """
         if len(X) > 0:
             if lc is None:
                 lc = next(self._color_seq)[self.color_mode]
-            self._plots += [Histogram.create(X, bins, lc)]
+            self._plots += [Histogram(X, bins, lc)]
 
-    def show(self, legend=False):
+    def text(self, X, Y, texts, lc=None):
+        """Plot texts at coordinates X, Y.
+
+        Always print the first character of a text at its
+        x, y coordinate and continue to the right. Character
+        extending the canvas are cut.
+
+        Parameters:
+            X: List[float]     X values.
+            Y: List[float]     Y values.
+            texts: List[str]   Texts to print. X, Y and texts must have the same
+                               number of entries.
+            lc: multiple       The (text) line color.
+        """
+        if len(X) > 0:
+            self._texts += [Text(X, Y, texts, lc)]
+
+    def axvline(self, x, ymin=0, ymax=1, lc=None):
+        """Plot a vertical line at x.
+
+        Parameters:
+            x: float       x-coordinate of the vertical line.
+                           In the range [0, 1]
+            ymin: float    Minimum y-coordinate of the vertical line.
+                           In the range [0, 1]
+            ymax: float    Maximum y-coordinate of the vertical line.
+                           In the range [0, 1]
+            lc: multiple   The line color.
+        """
+        self._spans.append(Span(x, x, ymin, ymax, lc))
+
+    def axvspan(self, xmin, xmax, ymin=0, ymax=1, lc=None):
+        """Plot a vertical rectangle from (xmin,ymin) to (xmax, ymax).
+
+        Parameters:
+            xmin: float    Minimum x-coordinate of the rectangle.
+                           In the range [0, 1]
+            xmax: float    Maximum x-coordinate of the rectangle.
+                           In the range [0, 1]
+            ymin: float    Minimum y-coordinate of the rectangle.
+                           In the range [0, 1]
+            ymax: float    Maximum y-coordinate of the rectangle.
+                           In the range [0, 1]
+            lc: multiple   The line color.
+        """
+        self._spans.append(Span(xmin, xmax, ymin, ymax, lc))
+
+    def axhline(self, y, xmin=0, xmax=1, lc=None):
+        """Plot a horizontal line at y.
+
+        Parameters:
+            y: float       y-coordinate of the horizontal line.
+                           In the range [0, 1]
+            x_min: float   Minimum x-coordinate of the vertical line.
+                           In the range [0, 1]
+            x_max: float   Maximum x-coordinate of the vertical line.
+                           In the range [0, 1]
+            lc: multiple   The line color.
+        """
+        self._spans.append(Span(xmin, xmax, y, y, lc))
+
+    def axhspan(self, ymin, ymax, xmin=0, xmax=1, lc=None):
+        """Plot a horizontal rectangle from (xmin,ymin) to (xmax, ymax).
+
+        Parameters:
+            ymin: float    Minimum y-coordinate of the rectangle.
+                           In the range [0, 1]
+            ymax: float    Maximum y-coordinate of the rectangle.
+                           In the range [0, 1]
+            xmin: float    Minimum x-coordinate of the rectangle.
+                           In the range [0, 1]
+            xmax: float    Maximum x-coordinate of the rectangle.
+                           In the range [0, 1]
+            lc: multiple   The line color.
+        """
+        self._spans.append(Span(xmin, xmax, ymin, ymax, lc))
+
+    def imgshow(self, X, cmap=None):
+        """Display data as an image, i.e., on a 2D regular raster.
+
+        Parameters:
+            X: array-like
+                The image data. Supported array shapes are:
+                - (M, N): an image with scalar data. The values are mapped
+                        to colors using a colormap. The values have to be in
+                        the 0-1 (float) range. Out of range, invalid type and
+                        None values are handled by the cmap.
+                - (M, N, 3): an image with RGB values (0-1 float or 0-255 int).
+
+                The first two dimensions (M, N) define the rows and columns of the image.
+
+            cmap: cmapstr or Colormap
+                The Colormap instance or registered colormap name used
+                to map scalar data to colors. This parameter is ignored
+                for RGB data.
+        """
+        if len(X) > 0:
+            self._heats += [Heat(X, cmap)]
+
+    def show(self, legend=False):  # noqa: C901 complex (12)
+        """Compute the plot.
+
+        Parameters:
+            legend: bool   Add the legend? default: False
+
+        Returns:
+            plot: str
+        """
         xmin, xmax = self.x_limits()
         ymin, ymax = self.y_limits()
-        if all(isinstance(p, Histogram) for p in self._plots):
+        if self._plots and all(isinstance(p, Histogram) for p in self._plots):
             ymin = 0
+
+        if self._heats and self._width is None and self._height is None:
+            self.height = len(self._heats[0].X)
+            self.width = len(self._heats[0].X[0])
+
         # create canvas
         canvas = Canvas(self.width, self.height,
                         self._in_fmt.convert(xmin), self._in_fmt.convert(ymin),
                         self._in_fmt.convert(xmax), self._in_fmt.convert(ymax),
-                        self.background, self.color_mode)
+                        self.background, **self._color_kwargs)
+
+        for s in self._spans:
+            s.write(canvas, self.with_colors)
 
         plot_origin = False
         for p in self._plots:
@@ -273,6 +480,12 @@ class Figure(object):
             if isinstance(p, Plot):
                 plot_origin = True
 
+        for t in self._texts:
+            t.write(canvas, self.with_colors, self._in_fmt)
+
+        for h in self._heats:
+            h.write(canvas)
+
         if self.origin and plot_origin:
             # print X / Y origin axis
             canvas.line(self._in_fmt.convert(xmin), 0, self._in_fmt.convert(xmax), 0)
@@ -296,86 +509,24 @@ class Figure(object):
         )
 
         if legend:
-            res += '\n\nLegend:\n-------\n'
-            res += '\n'.join([
-                color('⠤⠤ {}'.format(p.label if p.label is not None else 'Label {}'.format(i)),
-                      fg=p.lc, mode=self.color_mode, no_color=not self.with_colors)
-                for i, p in enumerate(self._plots)
-                if isinstance(p, Plot)
-            ])
+            res += '{0}{0}Legend:{0}-------{0}'.format(self.linesep)
+            lines = []
+            for i, p in enumerate(self._plots):
+                if isinstance(p, Plot):
+                    lbl = p.label or 'Label {}'.format(i)
+                    marker = p.marker or ''
+                    lines += [
+                        color(
+                            '⠤{}⠤ {}'.format(marker, lbl),
+                            fg=p.lc,
+                            mode=self.color_mode,
+                            no_color=not self.with_colors,
+                        ),
+                    ]
+            res += self.linesep.join(lines)
         return res
 
 
-class Plot(namedtuple('Plot', ['X', 'Y', 'lc', 'interp', 'label'])):
-
-    @classmethod
-    def create(cls, X, Y, lc, interp, label):
-        if len(X) != len(Y):
-            raise ValueError('X and Y dim have to be the same.')
-        if interp not in ('linear', None):
-            raise ValueError('Only "linear" and None are allowed values for `interp`.')
-
-        return cls(X, Y, lc, interp, label)
-
-    def width_vals(self):
-        return self.X
-
-    def height_vals(self):
-        return self.Y
-
-    def write(self, canvas, with_colors, in_fmt):
-        # make point iterators
-        from_points = zip(map(in_fmt.convert, self.X), map(in_fmt.convert, self.Y))
-        to_points = zip(map(in_fmt.convert, self.X), map(in_fmt.convert, self.Y))
-
-        # remove first point of to_points
-        (x0, y0) = next(to_points)
-
-        color = self.lc if with_colors else None
-
-        # print first point
-        canvas.point(x0, y0, color=color)
-
-        # plot other points and lines
-        for (x0, y0), (x, y) in zip(from_points, to_points):
-            canvas.point(x, y, color=color)
-            if self.interp == 'linear':
-                canvas.line(x0, y0, x, y, color=color)
-
-
-class Histogram(namedtuple('Histogram', ['X', 'bins', 'frequencies', 'buckets', 'lc'])):
-    @classmethod
-    def create(cls, X, bins, lc):
-        frequencies, buckets = hist(X, bins)
-
-        return cls(X, bins, frequencies, buckets, lc)
-
-    def width_vals(self):
-        return self.X
-
-    def height_vals(self):
-        return self.frequencies
-
-    def write(self, canvas, with_colors, in_fmt):
-        # how fat will one bar of the histogram be
-        x_diff = (canvas.dots_between(in_fmt.convert(self.buckets[0]), 0,
-                                      in_fmt.convert(self.buckets[1]), 0)[0] or 1)
-        bin_size = (in_fmt.convert(self.buckets[1]) - in_fmt.convert(self.buckets[0])) / x_diff
-
-        color = self.lc if with_colors else None
-        for i in range(self.bins):
-            # for each bucket
-            if self.frequencies[i] > 0:
-                for j in range(x_diff):
-                    # print bar
-                    x_ = in_fmt.convert(self.buckets[i]) + j * bin_size
-
-                    if canvas.xmin <= x_ <= canvas.xmax:
-                        canvas.line(x_, 0,
-                                    x_, self.frequencies[i],
-                                    color=color)
-
-
 def _limit(values):
     _min = 0
     _max = 1
