@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # The MIT License
 
-# Copyright (c) 2017 - 2018 Tammo Ippen, tammo.ippen@posteo.de
+# Copyright (c) 2017 - 2022 Tammo Ippen, tammo.ippen@posteo.de
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,24 +32,23 @@ from ._input_formatter import InputFormatter
 from ._util import hist as compute_hist
 
 
-def hist(X, bins=40, width=80, log_scale=False, linesep=os.linesep,
-         lc=None, bg=None, color_mode='names'):
-    """Create histogram over `X` from left to right
-
-    The values on the left are the center of the bucket, i.e. `(bin[i] + bin[i+1]) / 2`.
-    The values on the right are the total counts of this bucket.
+def hist_aggregated(counts, bins, width=80, log_scale=False, linesep=os.linesep,
+                    lc=None, bg=None, color_mode='names'):
+    """
+    Create histogram for aggregated data.
 
     Parameters:
-        X: List[float]  The items to count over.
-        bins: int       The number of bins to put X entries in (rows).
-        width: int      The number of characters for the width (columns).
-        log_scale: bool Scale the histogram with `log` function.
-        linesep: str    The requested line seperator. default: os.linesep
+        counts: List[int]    Counts for each bucket.
+        bins: List[float]    Limits for the bins for the provided counts: limits for
+                             bin `i` are `[bins[i], bins[i+1])`.
+                             Hence, `len(bins) == len(counts) + 1`.
+        width: int           The number of characters for the width (columns).
+        log_scale: bool      Scale the histogram with `log` function.
+        linesep: str         The requested line seperator. default: os.linesep
         lc: multiple         Give the line color.
         bg: multiple         Give the background color.
         color_mode: str      Specify color input mode; 'names' (default), 'byte' or 'rgb'
                              see plotille.color.__docs__
-
     Returns:
         str: histogram over `X` from left to right.
     """
@@ -58,14 +57,18 @@ def hist(X, bins=40, width=80, log_scale=False, linesep=os.linesep,
             return log(a)
         return a
 
+    h = counts
+    b = bins
+
     ipf = InputFormatter()
-    h, b = compute_hist(X, bins)
     h_max = _scale(max(h)) or 1
     delta = b[-1] - b[0]
 
+    bins_count = len(h)
+
     canvas = ['        bucket       | {} {}'.format('_' * width, 'Total Counts')]
     lasts = ['', '⠂', '⠆', '⠇', '⡇', '⡗', '⡷', '⡿']
-    for i in range(bins):
+    for i in range(bins_count):
         hight = int(width * 8 * _scale(h[i]) / h_max)
         canvas += ['[{}, {}) | {} {}'.format(
             ipf.fmt(b[i], delta=delta, chars=8, left=True),
@@ -77,6 +80,32 @@ def hist(X, bins=40, width=80, log_scale=False, linesep=os.linesep,
     return linesep.join(canvas)
 
 
+def hist(X, bins=40, width=80, log_scale=False, linesep=os.linesep,
+         lc=None, bg=None, color_mode='names'):
+    """Create histogram over `X` from left to right
+
+    The values on the left are the center of the bucket, i.e. `(bin[i] + bin[i+1]) / 2`.
+    The values on the right are the total counts of this bucket.
+
+    Parameters:
+        X: List[float]       The items to count over.
+        bins: int            The number of bins to put X entries in (rows).
+        width: int           The number of characters for the width (columns).
+        log_scale: bool      Scale the histogram with `log` function.
+        linesep: str         The requested line seperator. default: os.linesep
+        lc: multiple         Give the line color.
+        bg: multiple         Give the background color.
+        color_mode: str      Specify color input mode; 'names' (default), 'byte' or 'rgb'
+                             see plotille.color.__docs__
+
+    Returns:
+        str: histogram over `X` from left to right.
+    """
+    counts, bins = compute_hist(X, bins)
+    return hist_aggregated(counts=counts, bins=bins, width=width, log_scale=log_scale,
+                           linesep=linesep, lc=lc, bg=bg, color_mode=color_mode)
+
+
 def histogram(X, bins=160, width=80, height=40, X_label='X', Y_label='Counts', linesep=os.linesep,
               x_min=None, x_max=None, y_min=None, y_max=None,
               lc=None, bg=None, color_mode='names'):
@@ -87,12 +116,12 @@ def histogram(X, bins=160, width=80, height=40, X_label='X', Y_label='Counts', l
     corresponding frequency.
 
     Parameters:
-        X: List[float]  The items to count over.
-        bins: int       The number of bins to put X entries in (columns).
-        height: int     The number of characters for the height (rows).
-        X_label: str    Label for X-axis.
-        Y_label: str    Label for Y-axis. max 8 characters.
-        linesep: str    The requested line seperator. default: os.linesep
+        X: List[float]       The items to count over.
+        bins: int            The number of bins to put X entries in (columns).
+        height: int          The number of characters for the height (rows).
+        X_label: str         Label for X-axis.
+        Y_label: str         Label for Y-axis. max 8 characters.
+        linesep: str         The requested line seperator. default: os.linesep
         x_min, x_max: float  Limits for the displayed X values.
         y_min, y_max: float  Limits for the displayed Y values.
         lc: multiple         Give the line color.
@@ -130,7 +159,8 @@ def histogram(X, bins=160, width=80, height=40, X_label='X', Y_label='Counts', l
 
 def scatter(X, Y, width=80, height=40, X_label='X', Y_label='Y', linesep=os.linesep,
             x_min=None, x_max=None, y_min=None, y_max=None,
-            lc=None, bg=None, color_mode='names', origin=True):
+            lc=None, bg=None, color_mode='names', origin=True,
+            marker=None):
     """Create scatter plot with X , Y values
 
     Basically plotting without interpolation:
@@ -151,17 +181,19 @@ def scatter(X, Y, width=80, height=40, X_label='X', Y_label='Y', linesep=os.line
         color_mode: str      Specify color input mode; 'names' (default), 'byte' or 'rgb'
                              see plotille.color.__docs__
         origin: bool         Whether to print the origin. default: True
+        marker: str          Instead of braille dots set a marker char.
 
     Returns:
         str: scatter plot over `X`, `Y`.
     """
     return plot(X, Y, width, height, X_label, Y_label, linesep, None,
-                x_min, x_max, y_min, y_max, lc, bg, color_mode, origin)
+                x_min, x_max, y_min, y_max, lc, bg, color_mode, origin, marker)
 
 
 def plot(X, Y, width=80, height=40, X_label='X', Y_label='Y', linesep=os.linesep, interp='linear',
          x_min=None, x_max=None, y_min=None, y_max=None,
-         lc=None, bg=None, color_mode='names', origin=True):
+         lc=None, bg=None, color_mode='names', origin=True,
+         marker=None):
     """Create plot with X , Y values and linear interpolation between points
 
     Parameters:
@@ -180,6 +212,7 @@ def plot(X, Y, width=80, height=40, X_label='X', Y_label='Y', linesep=os.linesep
         color_mode: str        Specify color input mode; 'names' (default), 'byte' or 'rgb'
                                see plotille.color.__docs__
         origin: bool           Whether to print the origin. default: True
+        marker: str            Instead of braille dots set a marker char for actual values.
 
     Returns:
         str: plot over `X`, `Y`.
@@ -205,6 +238,6 @@ def plot(X, Y, width=80, height=40, X_label='X', Y_label='Y', linesep=os.linesep
     if lc is None and bg is None:
         fig.with_colors = False
 
-    fig.plot(X, Y, lc, interp)
+    fig.plot(X, Y, lc, interp, marker=marker)
 
     return fig.show()
