@@ -3,7 +3,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 # The MIT License
 
-# Copyright (c) 2017 - 2018 Tammo Ippen, tammo.ippen@posteo.de
+# Copyright (c) 2017 - 2022 Tammo Ippen, tammo.ippen@posteo.de
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,9 +31,6 @@ import time
 def roundeven(x):
     """Round to next even integer number in case of `X.5`
 
-    In Python3 this is the same as `round(x, 0)`, but since Python2 rounds up
-    in that case and I want consistent behaviour, here is the roundeven function.
-
     Parameters:
         x: float  The number to round.
 
@@ -44,10 +41,14 @@ def roundeven(x):
     """
     if math.isinf(x) or math.isnan(x):
         return x  # same behaviour as in python2
-    x_r = round(x)
-    if abs(x_r - x) == 0.5:
-        return int(2.0 * round(x / 2))
-    return x_r
+    return round(x)
+
+
+def _numpy_to_native(x):
+    # cf. https://numpy.org/doc/stable/reference/generated/numpy.ndarray.item.html
+    if ("<class 'numpy." in str(type(x)) or "<type 'numpy." in str(type(x))) and callable(x.item):
+        return x.item()
+    return x
 
 
 def hist(X, bins):
@@ -69,6 +70,10 @@ def hist(X, bins):
     if xmin == xmax:
         xmin -= 0.5
         xmax += 0.5
+
+    xmin = _numpy_to_native(xmin)
+    xmax = _numpy_to_native(xmax)
+
     delta = xmax - xmin
     is_datetime = False
     if isinstance(delta, timedelta):
@@ -79,7 +84,8 @@ def hist(X, bins):
 
     y = [0] * bins
     for x in X:
-        delta = (x - xmin)
+        x_ = _numpy_to_native(x)
+        delta = (x_ - xmin)
         if isinstance(delta, timedelta):
             delta = timestamp(delta)
         x_idx = min(bins - 1, int(delta // xwidth))
